@@ -10,6 +10,7 @@ use criterion::{
 mod meshes;
 mod primitives;
 
+use forsyth::Config;
 use meshes::*;
 use primitives::*;
 
@@ -107,8 +108,8 @@ fn bench_fan(c: &mut Criterion) {
 
     for input in &bench_inputs {
         is_permutated_mesh(
-            &input.as_slice(),
-            forsyth::order_triangles(&input.as_slice(), 32)
+            input.as_slice(),
+            forsyth::order_triangles(input.as_slice())
                 .unwrap()
                 .as_slice(),
         )
@@ -119,11 +120,7 @@ fn bench_fan(c: &mut Criterion) {
             BenchmarkId::from_parameter(input.len() / 3),
             input,
             |b, input| {
-                b.iter(|| {
-                    forsyth::order_triangles(&input, black_box(32))
-                        .unwrap()
-                        .len()
-                });
+                b.iter(|| forsyth::order_triangles(input).unwrap().len());
             },
         );
     }
@@ -141,8 +138,8 @@ fn bench_multi_fans(c: &mut Criterion) {
 
     for input in &bench_inputs {
         is_permutated_mesh(
-            &input.as_slice(),
-            forsyth::order_triangles(&input.as_slice(), 32)
+            input.as_slice(),
+            forsyth::order_triangles(input.as_slice())
                 .unwrap()
                 .as_slice(),
         )
@@ -153,11 +150,7 @@ fn bench_multi_fans(c: &mut Criterion) {
             BenchmarkId::from_parameter(input.len() / 3),
             input,
             |b, input| {
-                b.iter(|| {
-                    forsyth::order_triangles(&input, black_box(32))
-                        .unwrap()
-                        .len()
-                });
+                b.iter(|| forsyth::order_triangles(input).unwrap().len());
             },
         );
     }
@@ -178,8 +171,8 @@ fn bench_quad_patch(c: &mut Criterion) {
 
     for input in &bench_inputs {
         is_permutated_mesh(
-            &input.as_slice(),
-            forsyth::order_triangles(&input.as_slice(), 32)
+            input.as_slice(),
+            forsyth::order_triangles(input.as_slice())
                 .unwrap()
                 .as_slice(),
         )
@@ -190,11 +183,7 @@ fn bench_quad_patch(c: &mut Criterion) {
             BenchmarkId::from_parameter(input.len() / 3),
             input,
             |b, input| {
-                b.iter(|| {
-                    forsyth::order_triangles(&input, black_box(32))
-                        .unwrap()
-                        .len()
-                });
+                b.iter(|| forsyth::order_triangles(input).unwrap().len());
             },
         );
     }
@@ -253,11 +242,11 @@ fn bench_caches(c: &mut Criterion) {
         let mut bench_inputs: Vec<(String, Vec<u32>)> = bench_sources
             .iter()
             .map(|src| {
-                let indices = generate_mesh_indices(&src.1);
+                let indices = generate_mesh_indices(src.1);
 
                 is_permutated_mesh(
                     &indices,
-                    forsyth::order_triangles(&indices, 32).unwrap().as_slice(),
+                    forsyth::order_triangles(&indices).unwrap().as_slice(),
                 )
                 .unwrap();
 
@@ -277,7 +266,7 @@ fn bench_caches(c: &mut Criterion) {
     for cache_size in &[16_u16, 32, 64] {
         for input in &bench_inputs {
             let mesh_name = &input.0;
-            let mesh_indices = input.1.as_slice();
+            let mesh_indices = &input.1;
 
             group.throughput(Throughput::Elements(mesh_indices.len() as u64));
 
@@ -286,9 +275,14 @@ fn bench_caches(c: &mut Criterion) {
                 mesh_indices,
                 |b, mesh_indices| {
                     b.iter(|| {
-                        forsyth::order_triangles(mesh_indices, black_box(*cache_size))
-                            .unwrap()
-                            .len()
+                        let mut mesh_indices = mesh_indices.clone();
+                        forsyth::order_triangles_inplace(
+                            Config::default(),
+                            mesh_indices.as_mut_slice(),
+                            black_box(*cache_size),
+                        )
+                        .unwrap()
+                        .len()
                     });
                 },
             );
@@ -361,7 +355,7 @@ fn bench_meshes(c: &mut Criterion) {
             .map(|src| {
                 let indices = src.1.clone();
 
-                let ordered = forsyth::order_triangles(&indices, 32).unwrap();
+                let ordered = forsyth::order_triangles(&indices).unwrap();
 
                 is_permutated_mesh(&indices, ordered.as_slice()).unwrap();
 
@@ -412,7 +406,7 @@ fn bench_meshes(c: &mut Criterion) {
                 let seed = (seed & 0xFFFF) as u16;
                 BenchVertex::new(seed)
             });
-            let mesh_indices_ordered = forsyth::order_triangles(mesh_indices, 32).unwrap();
+            let mesh_indices_ordered = forsyth::order_triangles(mesh_indices).unwrap();
             let fully_ordered_mesh =
                 forsyth::order_vertices(&mesh_vertices, &mesh_indices_ordered).unwrap();
 
@@ -533,11 +527,7 @@ fn bench_meshes(c: &mut Criterion) {
                 BenchmarkId::from_parameter(mesh_name),
                 mesh_indices,
                 |b, mesh_indices| {
-                    b.iter(|| {
-                        forsyth::order_triangles(mesh_indices, black_box(32))
-                            .unwrap()
-                            .len()
-                    });
+                    b.iter(|| forsyth::order_triangles(mesh_indices).unwrap().len());
                 },
             );
         }
