@@ -1,3 +1,6 @@
+#![deny(clippy::all)]
+#![forbid(unsafe_code)]
+
 //! This is a clean-room implementation of Tom Forsyth's
 //! [`Linear-speed Vertex Cache Optimisation`] in Rust.
 //!
@@ -25,11 +28,15 @@
 //! let input_indices = &[0_u32, 1, 2, 0, 1, 3, 0, 3, 4, 2, 1, 4];
 //!
 //! // order indices first
-//! let ordered_indices = order_triangles(input_indices, 32).unwrap();
+//! let ordered_indices =
+//!     order_triangles(input_indices).unwrap_or_else(|_| input_indices.to_vec());
+//!
+//! assert_eq!(&ordered_indices, &[0, 3, 4, 0, 1, 3, 2, 1, 4, 0, 1, 2]);
 //!
 //! // then order vertices and remap indices accordingly
 //! let (ordered_vertices, ordered_indices) =
-//!     order_vertices(&input_vertices[..], ordered_indices.as_slice()).unwrap();
+//!     order_vertices(&input_vertices[..], ordered_indices.as_slice())
+//!         .unwrap_or_else(|_| (input_vertices.to_vec(), ordered_indices));
 //!
 //! assert_eq!(&ordered_vertices, &['a', 'd', 'e', 'b', 'c']);
 //! assert_eq!(&ordered_indices, &[0, 1, 2, 0, 3, 1, 4, 3, 2, 0, 3, 4]);
@@ -60,17 +67,12 @@ use std::{
     u32,
 };
 
-const FIND_VERTEX_SCORE_CACHE_DECAY_POWER: f32 = 1.5;
-const FIND_VERTEX_SCORE_LAST_TRI_SCORE: f32 = 0.75;
-const FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE: f32 = 2.0;
-const FIND_VERTEX_SCORE_VALENCE_BOOST_POWER: f32 = 0.5;
-
 const MAX_VERTEX_CACHE_SIZE: u16 = std::u16::MAX - 1;
 const NOT_IN_CACHE: u16 = std::u16::MAX;
 const NULL_TRI: u32 = std::u32::MAX;
 
 /// An error or invariant violation during an ordering operation
-#[derive(Debug, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq)]
 pub enum Error {
     /// Provided slice of Index cannot represent one or more index triples.
     IndicesNotTriples,
@@ -82,13 +84,15 @@ pub enum Error {
     TooManyTrianglesAtVertex { vertex_idx: usize, limit: usize },
     /// The resulting ordered Index draw list was malformed.
     MalformedDrawList,
+    /// There was an attempt to refer to an invalid vertex index
+    VertexOutOfBounds,
 }
 
 impl Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Error::IndicesNotTriples => f.write_str("elements are not all triples"),
-            Error::IndexToUsizeConversion => f.write_str("can not convert Index to usize"),
+            Error::IndexToUsizeConversion => f.write_str("cannot convert Index to usize"),
             Error::TooManyTrianglesInTotal { limit } => f.write_fmt(format_args!(
                 "too many triangles in total. {} triangles are supported",
                 limit,
@@ -100,6 +104,33 @@ impl Display for Error {
             Error::MalformedDrawList => {
                 f.write_str("the generated ordered Index draw list is malformed")
             }
+            Error::VertexOutOfBounds => f.write_str("the vertex index is out of bounds"),
+        }
+    }
+}
+
+/// A struct holding the parameters controlling the algorithm
+///
+/// Using Config::default() results in a configuration with
+/// the parameters set as suggested by the original paper.
+///
+/// See the paper for further information.
+
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub struct Config {
+    pub cache_decay_power: f32,
+    pub last_tri_score: f32,
+    pub valence_boost_scale: f32,
+    pub valence_boost_power: f32,
+}
+
+impl Default for Config {
+    fn default() -> Self {
+        Self {
+            cache_decay_power: 1.5,
+            last_tri_score: 0.75,
+            valence_boost_scale: 2.0,
+            valence_boost_power: 0.5,
         }
     }
 }
@@ -146,13 +177,18 @@ struct TriangleInfo<Index> {
     added_to_draw_list: bool,
 }
 
-fn calculate_vertex_score(num_active_tris: u32, cache_pos: u16, vertex_cache_size: u16) -> f32 {
+fn calculate_vertex_score(
+    config: &Config,
+    num_active_tris: u32,
+    cache_pos: u16,
+    vertex_cache_size: u16,
+) -> f32 {
     if num_active_tris == 0 {
         -1.0f32
     } else {
         // TF: Bonus points for having a low number of tris still to
         // use the vert, so we get rid of lone verts quickly.
-        let valence_boost = (num_active_tris as f32).powf(-FIND_VERTEX_SCORE_VALENCE_BOOST_POWER);
+        let valence_boost = (num_active_tris as f32).powf(-config.valence_boost_power);
 
         let pos_score = if cache_pos < 3 {
             // TF: This vertex was used in the last triangle,
@@ -160,40 +196,44 @@ fn calculate_vertex_score(num_active_tris: u32, cache_pos: u16, vertex_cache_siz
             // it's in. Otherwise, you can get very different
             // answers depending on whether you add
             // the triangle 1,2,3 or 3,1,2 - which is silly.
-            FIND_VERTEX_SCORE_LAST_TRI_SCORE
+            config.last_tri_score
         } else if cache_pos < vertex_cache_size {
             // TF: Points for being high in the cache.
             let scaler = 1.0 / ((vertex_cache_size - 3) as f32);
-            (1.0 - ((cache_pos - 3) as f32) * scaler).powf(FIND_VERTEX_SCORE_CACHE_DECAY_POWER)
+            (1.0 - ((cache_pos - 3) as f32) * scaler).powf(config.cache_decay_power)
         } else {
             0.0f32
         };
 
-        pos_score + (FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE * valence_boost)
+        pos_score + (config.valence_boost_scale * valence_boost)
     }
 }
 
 /// Orders a slice of triangle indices in place.
 ///
+/// The configuration in `config` can be used to tweak the algorithm's parameters.
+/// No checks for sanity and/or validity of the provided values is done.
+///
 /// The slice `indices` is assumed to contain the index triples making
 /// up the triangles to be ordered.
 ///
 /// `vertex_cache_size` controls the size of the simulated LRU cache that
-/// is used to drive the optimization, with often-used value of 32.
+/// is used to drive the optimization. A size often used is 32.
 ///
 /// Smaller cache sizes result in faster but worse ordering, whereas larger size
 /// increase hit rates at the cost of runtime.
 /// [`This size-performance-relationship may be sub-linear`]
-/// The size is clamped internally to implementation-specific ranges.
+/// Cache size is clamped internally to implementation-specific ranges.
+///
 ///
 /// Returns a reference to `indices`
 ///
 /// ```
-/// use forsyth::order_triangles_inplace;
+/// use forsyth::{Config, order_triangles_inplace};
 ///
 /// let mut indices = [0_u16, 1, 2, 3, 2, 0, 0, 1, 3, 0, 1, 4, 0, 1, 5];
 /// assert!(
-///     order_triangles_inplace(&mut indices, 32) ==
+///     order_triangles_inplace(Config::default(), &mut indices, 32) ==
 ///     Ok(&[0_u16, 1, 4, 0, 1, 5, 0, 1, 2, 3, 2, 0, 0, 1, 3])
 /// );
 /// ```
@@ -201,16 +241,12 @@ fn calculate_vertex_score(num_active_tris: u32, cache_pos: u16, vertex_cache_siz
 /// [`This size-performance-relationship may be sub-linear`]: https://kento_asashima.gitlab.io/-/forsyth/-/jobs/1192073834/artifacts/target/criterion/caches/report/index.html
 
 pub fn order_triangles_inplace<Index>(
+    config: Config,
     indices: &mut [Index],
     vertex_cache_size: u16,
 ) -> Result<&[Index], Error>
 where
-    Index: TryInto<usize>
-        + TryFrom<usize>
-        + std::clone::Clone
-        + std::default::Default
-        + std::cmp::PartialEq
-        + Copy,
+    Index: TryInto<usize> + std::default::Default + std::cmp::PartialEq + Copy,
 {
     let vertex_cache_size =
         std::cmp::max(4, std::cmp::min(vertex_cache_size, MAX_VERTEX_CACHE_SIZE));
@@ -307,8 +343,12 @@ where
     let mut best_score_tri = (NULL_TRI, 0.0_f32);
 
     for vert in &mut verts {
-        vert.score =
-            calculate_vertex_score(vert.num_tris_active as u32, NOT_IN_CACHE, vertex_cache_size);
+        vert.score = calculate_vertex_score(
+            &config,
+            vert.num_tris_active as u32,
+            NOT_IN_CACHE,
+            vertex_cache_size,
+        );
         for i in 0..vert.num_tris_active {
             let tri_idx = vertex_tri_list[vert.tri_list_ofs as usize + i as usize];
             let mut tri = &mut tris[tri_idx as usize];
@@ -401,6 +441,7 @@ where
                 };
                 let old_score = vert.score;
                 vert.score = calculate_vertex_score(
+                    &config,
                     vert.num_tris_active as u32,
                     cache_pos,
                     vertex_cache_size,
@@ -481,88 +522,85 @@ where
 /// A convenience function that wraps [`order_triangles_in_place`] to return a newly
 /// allocated ordered index buffer.
 ///
+/// This is equivalent to calling [`order_triangles_in_place`] with default [`Config`] and `cache_size` of 32.
+///
 /// ```
 /// use forsyth::{Error,order_triangles};
 ///
 /// // a triangle
-/// assert_eq!(order_triangles(&[0, 1, 2], 32), Ok(vec![0, 1, 2]));
+/// assert_eq!(order_triangles(&[0, 1, 2]), Ok(vec![0, 1, 2]));
 /// // a quad
 /// assert_eq!(
-///     order_triangles(&[0_u32, 1, 2, 0, 2, 3], 32),
+///     order_triangles(&[0_u32, 1, 2, 0, 2, 3]),
 ///     Ok(vec![0, 1, 2, 0, 2, 3]),
 /// );
 ///
 /// assert_eq!(
-///     order_triangles(&Vec::<u32>::new(), 32),
+///     order_triangles(&Vec::<u32>::new()),
 ///     Err(Error::IndicesNotTriples)
 /// );
 /// assert_eq!(
-///     order_triangles(&[0, 1], 32),
+///     order_triangles(&[0, 1]),
 ///     Err(Error::IndicesNotTriples)
 /// );
 /// assert_eq!(
-///     order_triangles(&[0, 1, 2, 3], 32),
+///     order_triangles(&[0, 1, 2, 3]),
 ///     Err(Error::IndicesNotTriples)
 /// );
 ///
 /// // Works with all (un-)signed integers
 /// assert_eq!(
-///     order_triangles(&[0_u8, 1, 2, 3, 4, 5], 32),
+///     order_triangles(&[0_u8, 1, 2, 3, 4, 5]),
 ///     Ok(vec![0, 1, 2, 3, 4, 5])
 /// );
 /// assert_eq!(
-///     order_triangles(&[0_i16, 1, 2, 3, 4, 5], 32),
+///     order_triangles(&[0_i16, 1, 2, 3, 4, 5]),
 ///     Ok(vec![0, 1, 2, 3, 4, 5])
 /// );
 /// assert_eq!(
-///     order_triangles(&[0_u32, 1, 2, 3, 4, 5], 32),
+///     order_triangles(&[0_u32, 1, 2, 3, 4, 5]),
 ///     Ok(vec![0, 1, 2, 3, 4, 5])
 /// );
 /// assert_eq!(
-///     order_triangles(&[0_i64, 1, 2, 3, 4, 5], 32),
+///     order_triangles(&[0_i64, 1, 2, 3, 4, 5]),
 ///     Ok(vec![0, 1, 2, 3, 4, 5])
 /// );
 /// assert_eq!(
-///     order_triangles(&[0_usize, 1, 2, 3, 4, 5], 32),
+///     order_triangles(&[0_usize, 1, 2, 3, 4, 5]),
 ///     Ok(vec![0, 1, 2, 3, 4, 5])
 /// );
 ///
 /// // Indices can be sparse
 /// assert_eq!(
-///     order_triangles(&[0_i8, 1, 2, 63, 64, 65], 32),
+///     order_triangles(&[0_i8, 1, 2, 63, 64, 65]),
 ///     Ok(vec![0, 1, 2, 63, 64, 65])
 /// );
 ///
 /// // Indices cannot be negative
 /// assert_eq!(
-///    order_triangles(&[0_i8, -1, -2, 63, 64, 65], 32),
+///    order_triangles(&[0_i8, -1, -2, 63, 64, 65]),
 ///    Err(Error::IndexToUsizeConversion)
 /// );
 /// ```
 ///
 /// [`order_triangles_in_place`]: ./fn.order_triangles_inplace.html
+/// [`Config`]: ./struct.Config.html
 
-pub fn order_triangles<Index>(
-    indices: &[Index],
-    vertex_cache_size: u16,
-) -> Result<Vec<Index>, Error>
+pub fn order_triangles<Index>(indices: &[Index]) -> Result<Vec<Index>, Error>
 where
-    Index: std::convert::TryInto<usize>
-        + std::convert::TryFrom<usize>
-        + std::clone::Clone
-        + std::default::Default
-        + std::cmp::PartialEq
-        + Copy,
+    Index: std::convert::TryInto<usize> + std::default::Default + std::cmp::PartialEq + Copy,
 {
     if indices.is_empty() || (indices.len() % 3) != 0 {
         return Err(Error::IndicesNotTriples);
     }
 
+    let vertex_cache_size = 32;
+
     let buffer = {
         let mut buffer = Vec::with_capacity(indices.len());
         buffer.extend_from_slice(indices);
 
-        order_triangles_inplace(buffer.as_mut_slice(), vertex_cache_size)?;
+        order_triangles_inplace(Config::default(), buffer.as_mut_slice(), vertex_cache_size)?;
 
         buffer
     };
@@ -576,7 +614,7 @@ where
 ///
 /// This function can also be used to consolidate sparse index and vertex data as unreferenced vertices are not copied.
 /// ```
-/// use forsyth::order_vertices;
+/// use forsyth::{Error,order_vertices};
 ///
 /// assert_eq!(
 ///     order_vertices(&['d', 'c', 'b', 'a'], &[3, 2, 0, 2, 1, 0]),
@@ -592,11 +630,11 @@ where
 /// // Indices must be in valid range
 /// assert_eq!(
 ///     order_vertices(&['a', 'b', 'c'], &[0_i8, 1, 99]),
-///     Err("index does not point into vertex array (out-of-bounds)".to_string())
+///     Err(Error::VertexOutOfBounds)
 /// );
 /// assert_eq!(
 ///     order_vertices(&['a', 'b', 'c'], &[0_i8, -1, 2]),
-///     Err("failed to convert Index to usize".to_string())
+///     Err(Error::IndexToUsizeConversion)
 /// );
 
 /// ```
@@ -605,7 +643,7 @@ where
 pub fn order_vertices<Index, Vertex>(
     vertices: &[Vertex],
     indices: &[Index],
-) -> Result<(Vec<Vertex>, Vec<Index>), String>
+) -> Result<(Vec<Vertex>, Vec<Index>), Error>
 where
     Index: Copy + Eq + TryInto<usize> + TryFrom<usize> + std::hash::Hash,
     Vertex: Copy,
@@ -620,17 +658,15 @@ where
             std::collections::hash_map::Entry::Vacant(vacant) => {
                 let index = (*index)
                     .try_into()
-                    .map_err(|_| "failed to convert Index to usize".to_string())?;
+                    .map_err(|_| Error::IndexToUsizeConversion)?;
                 if let Some(vertex) = vertices.get(index) {
                     ordered_vertices.push(*vertex);
                     let mapped = (ordered_vertices.len() - 1)
                         .try_into()
-                        .map_err(|_| "error mapping index".to_string())?;
+                        .map_err(|_| Error::IndexToUsizeConversion)?;
                     *vacant.insert(mapped)
                 } else {
-                    return Err(
-                        "index does not point into vertex array (out-of-bounds)".to_string()
-                    );
+                    return Err(Error::VertexOutOfBounds);
                 }
             }
         };
@@ -645,17 +681,65 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn error_formatting() {
+        for e in [
+            Error::IndexToUsizeConversion,
+            Error::IndicesNotTriples,
+            Error::TooManyTrianglesAtVertex {
+                vertex_idx: 123,
+                limit: 42,
+            },
+            Error::TooManyTrianglesInTotal { limit: 42 },
+            Error::MalformedDrawList,
+            Error::VertexOutOfBounds,
+        ] {
+            assert_eq!(
+                format!("{}", e),
+                match e {
+                    Error::IndicesNotTriples => "elements are not all triples",
+                    Error::IndexToUsizeConversion => "cannot convert Index to usize",
+                    Error::TooManyTrianglesInTotal { .. } =>
+                        "too many triangles in total. 42 triangles are supported",
+                    Error::TooManyTrianglesAtVertex { .. } =>
+                        "too many triangles connected to vertex 123. 42 triangles are supported",
+                    Error::MalformedDrawList =>
+                        "the generated ordered Index draw list is malformed",
+                    Error::VertexOutOfBounds => "the vertex index is out of bounds",
+                }
+                .to_string()
+            );
+        }
+    }
+
+    #[test]
+    fn config() {
+        assert_eq!(
+            Config {
+                cache_decay_power: 1.5,
+                last_tri_score: 0.75,
+                valence_boost_scale: 2.0,
+                valence_boost_power: 0.5,
+            },
+            Config::default()
+        );
+    }
+
     #[test]
     fn combined() {
         let input_vertices = &['a', 'b', 'c', 'd', 'e'];
         let input_indices = &[0_u32, 1, 2, 0, 1, 3, 0, 3, 4, 2, 1, 4];
 
         // order indices first
-        let ordered_indices = order_triangles(input_indices, 32).unwrap();
+        let ordered_indices =
+            order_triangles(input_indices).unwrap_or_else(|_| input_indices.to_vec());
+
+        assert_eq!(&ordered_indices, &[0, 3, 4, 0, 1, 3, 2, 1, 4, 0, 1, 2]);
 
         // then order vertices and remap indices accordingly
         let (ordered_vertices, ordered_indices) =
-            order_vertices(&input_vertices[..], ordered_indices.as_slice()).unwrap();
+            order_vertices(&input_vertices[..], ordered_indices.as_slice())
+                .unwrap_or_else(|_| (input_vertices.to_vec(), ordered_indices));
 
         assert_eq!(&ordered_vertices, &['a', 'd', 'e', 'b', 'c']);
         assert_eq!(&ordered_indices, &[0, 1, 2, 0, 3, 1, 4, 3, 2, 0, 3, 4]);
@@ -665,7 +749,7 @@ mod tests {
     fn fuzz_regressions() {
         {
             assert_eq!(
-                order_triangles(&[i16::from_be_bytes([130_u8, 246])], 9),
+                order_triangles(&[i16::from_be_bytes([130_u8, 246])]),
                 Err(Error::IndicesNotTriples)
             );
         }
@@ -688,38 +772,26 @@ mod tests {
     fn test_vertex_score() {
         const MAX_VERTEX_CACHE_SIZE: u16 = 32;
 
+        let config = Config::default();
+
         for (cache_pos, ref_score) in [
-            (-2, FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE),
-            (-1, FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE),
-            (
-                0,
-                FIND_VERTEX_SCORE_LAST_TRI_SCORE + FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE,
-            ),
-            (
-                1,
-                FIND_VERTEX_SCORE_LAST_TRI_SCORE + FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE,
-            ),
-            (
-                2,
-                FIND_VERTEX_SCORE_LAST_TRI_SCORE + FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE,
-            ),
+            (-2, config.valence_boost_scale),
+            (-1, config.valence_boost_scale),
+            (0, config.last_tri_score + config.valence_boost_scale),
+            (1, config.last_tri_score + config.valence_boost_scale),
+            (2, config.last_tri_score + config.valence_boost_scale),
             (3, 3.0),
             (4, 2.9487243),
             (5, 2.8983564),
             (6, 2.8489127),
             (MAX_VERTEX_CACHE_SIZE as i32 - 1, 2.0064032),
-            (
-                MAX_VERTEX_CACHE_SIZE as i32,
-                FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE,
-            ),
-            (
-                2 * MAX_VERTEX_CACHE_SIZE as i32,
-                FIND_VERTEX_SCORE_VALENCE_BOOST_SCALE,
-            ),
+            (MAX_VERTEX_CACHE_SIZE as i32, config.valence_boost_scale),
+            (2 * MAX_VERTEX_CACHE_SIZE as i32, config.valence_boost_scale),
         ]
         .iter()
         {
             let score = calculate_vertex_score(
+                &config,
                 1,
                 if *cache_pos < 0 {
                     MAX_VERTEX_CACHE_SIZE + 1
@@ -744,87 +816,99 @@ mod tests {
         {
             let mut indices = [0_u16, 1, 2, 3, 2, 0, 0, 1, 3, 0, 1, 4, 0, 1, 5];
             assert_eq!(
-                order_triangles_inplace(&mut indices, 32).unwrap(),
+                order_triangles_inplace(Config::default(), &mut indices, 32).unwrap(),
                 &[0, 1, 4, 0, 1, 5, 0, 1, 2, 3, 2, 0, 0, 1, 3]
             );
         }
 
         assert_eq!(
-            order_triangles_inplace::<u32>(&mut [], 32),
+            order_triangles_inplace::<u32>(Config::default(), &mut [], 32),
             Err(Error::IndexToUsizeConversion)
         );
 
         assert_eq!(
-            order_triangles_inplace(&mut [-1_i32, 0, 2], 32),
+            order_triangles_inplace(Config::default(), &mut [-1_i32, 0, 2], 32),
             Err(Error::IndexToUsizeConversion)
         );
 
         // 1 shared edge - quad
         assert_eq!(
-            order_triangles(&[0_u32, 1, 2, 0, 2, 3], 32).unwrap(),
+            order_triangles(&[0_u32, 1, 2, 0, 2, 3]).unwrap(),
             [0, 1, 2, 0, 2, 3]
         );
 
         assert_eq!(
-            order_triangles(&Vec::<u32>::new(), 32),
+            order_triangles(&Vec::<u32>::new()),
             Err(Error::IndicesNotTriples)
         );
-        assert_eq!(order_triangles(&[0], 32), Err(Error::IndicesNotTriples));
-        assert_eq!(order_triangles(&[0, 1], 32), Err(Error::IndicesNotTriples));
-        assert_eq!(order_triangles(&[0, 1, 2], 32).unwrap(), [0, 1, 2]);
+        assert_eq!(order_triangles(&[0]), Err(Error::IndicesNotTriples));
+        assert_eq!(order_triangles(&[0, 1]), Err(Error::IndicesNotTriples));
+        assert_eq!(order_triangles(&[0, 1, 2]).unwrap(), [0, 1, 2]);
         assert_eq!(
-            order_triangles(&[0, 1, 2, 3], 32),
+            order_triangles(&[0, 1, 2, 3]),
             Err(Error::IndicesNotTriples)
         );
 
         // Two disjunctive tris in multiple types
         assert_eq!(
-            order_triangles(&[0_u32, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_u32, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_u16, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_u16, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_u8, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_u8, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_u64, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_u64, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
 
         assert_eq!(
-            order_triangles(&[0_i32, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_i32, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_i16, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_i16, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_i8, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_i8, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_i64, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_i64, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
         assert_eq!(
-            order_triangles(&[0_usize, 1, 2, 3, 4, 5], 32).unwrap(),
+            order_triangles(&[0_usize, 1, 2, 3, 4, 5]).unwrap(),
+            [0, 1, 2, 3, 4, 5]
+        );
+        assert_eq!(
+            order_triangles(&[0_isize, 1, 2, 3, 4, 5]).unwrap(),
+            [0, 1, 2, 3, 4, 5]
+        );
+        assert_eq!(
+            order_triangles(&[0_u128, 1, 2, 3, 4, 5]).unwrap(),
+            [0, 1, 2, 3, 4, 5]
+        );
+        assert_eq!(
+            order_triangles(&[0_i128, 1, 2, 3, 4, 5]).unwrap(),
             [0, 1, 2, 3, 4, 5]
         );
 
         // Indices are sparse
         assert_eq!(
-            order_triangles(&[0_i8, 1, 2, 63, 64, 65], 32).unwrap(),
+            order_triangles(&[0_i8, 1, 2, 63, 64, 65]).unwrap(),
             [0, 1, 2, 63, 64, 65]
         );
 
         // Indices are negative
         assert_eq!(
-            order_triangles(&[0_i8, -1, -2, 63, 64, 65], 32),
+            order_triangles(&[0_i8, -1, -2, 63, 64, 65]),
             Err(Error::IndexToUsizeConversion)
         );
 
@@ -851,7 +935,8 @@ mod tests {
                 std::u16::MAX - 1,
                 std::u16::MAX,
             ] {
-                let result = order_triangles(indices.as_slice(), cs);
+                let mut indices = indices.clone();
+                let result = order_triangles_inplace(Config::default(), indices.as_mut_slice(), cs);
                 assert!(result.is_ok(), "result is {:?}", result);
             }
         }
@@ -865,7 +950,7 @@ mod tests {
             }
 
             assert_eq!(
-                order_triangles(indices.as_slice(), 32),
+                order_triangles(indices.as_slice()),
                 Err(Error::TooManyTrianglesAtVertex {
                     vertex_idx: 0,
                     limit: 65535
@@ -888,12 +973,12 @@ mod tests {
 
         assert_eq!(
             order_vertices(&['a', 'b', 'c'], &[-1, 0, 2]),
-            Err("failed to convert Index to usize".to_string())
+            Err(Error::IndexToUsizeConversion)
         );
 
         assert_eq!(
             order_vertices(&['a', 'b', 'c'], &[0, 1, 3]),
-            Err("index does not point into vertex array (out-of-bounds)".to_string())
+            Err(Error::VertexOutOfBounds)
         );
     }
 }
